import {
  Body,
  Controller,
  Delete,
  Get,
  Patch,
  Path,
  Post,
  Request,
  Route,
  Security,
  Tags,
} from "tsoa";
import { AppDataSource } from "../database/data-source";
import {
  ProfileLeaveHistory,
  CreateProfileEmployeeLeave,
  ProfileLeave,
  UpdateProfileLeave,
} from "../entities/ProfileLeave";
import HttpSuccess from "../interfaces/http-success";
import HttpStatus from "../interfaces/http-status";
import HttpError from "../interfaces/http-error";
import { RequestWithUser } from "../middlewares/user";
import { LeaveType } from "../entities/LeaveType";
import { ProfileEmployee } from "../entities/ProfileEmployee";
import permission from "../interfaces/permission";
import { setLogDataDiff } from "../interfaces/utils";
@Route("api/v1/org/profile-temp/leave")
@Tags("ProfileLeave")
@Security("bearerAuth")
export class ProfileLeaveEmployeeTempController extends Controller {
  private profileRepo = AppDataSource.getRepository(ProfileEmployee);
  private leaveRepo = AppDataSource.getRepository(ProfileLeave);
  private leaveHistoryRepo = AppDataSource.getRepository(ProfileLeaveHistory);
  private leaveTypeRepository = AppDataSource.getRepository(LeaveType);

  @Get("user")
  public async getLeaveUser(@Request() request: { user: Record<string, any> }) {
    const profile = await this.profileRepo.findOneBy({ keycloak: request.user.sub });
    if (!profile) {
      throw new HttpError(HttpStatus.BAD_REQUEST, "ไม่พบ profile ดังกล่าว");
    }
    const record = await this.leaveRepo.find({
      relations: { leaveType: true },
      where: { profileEmployeeId: profile.id, isDeleted: false },
      order: { createdAt: "ASC" },
    });
    return new HttpSuccess(record);
  }

  @Get("{profileId}")
  public async getLeave(@Path() profileId: string, @Request() req: RequestWithUser) {
    let _workflow = await new permission().Workflow(req, profileId, "SYS_REGISTRY_TEMP");
    if (_workflow == false) await new permission().PermissionGet(req, "SYS_REGISTRY_TEMP");
    const record = await this.leaveRepo.find({
      relations: { leaveType: true },
      where: { profileEmployeeId: profileId, isDeleted: false },
      order: { createdAt: "ASC" },
    });
    return new HttpSuccess(record);
  }

  @Get("admin/{profileId}")
  public async getLeaveAdmin(@Path() profileId: string, @Request() req: RequestWithUser) {
    let _workflow = await new permission().Workflow(req, profileId, "SYS_WAGE");
    if (_workflow == false) await new permission().PermissionGet(req, "SYS_WAGE");
    const record = await this.leaveRepo.find({
      relations: { leaveType: true },
      where: { profileEmployeeId: profileId, isDeleted: false },
      order: { createdAt: "ASC" },
    });
    return new HttpSuccess(record);
  }

  @Get("admin/history/{leaveId}")
  public async leaveAdminHistory(@Path() leaveId: string, @Request() req: RequestWithUser) {
    let _workflow = await new permission().Workflow(req, leaveId, "SYS_REGISTRY_TEMP");
    if (_workflow == false) await new permission().PermissionGet(req, "SYS_REGISTRY_TEMP");
    const record = await this.leaveHistoryRepo.find({
      relations: { leaveType: true },
      where: { profileLeaveId: leaveId },
      order: { createdAt: "DESC" },
    });
    return new HttpSuccess(record);
  }

  @Get("history/{leaveId}")
  public async leaveHistory(@Path() leaveId: string) {
    const record = await this.leaveHistoryRepo.find({
      relations: { leaveType: true },
      where: { profileLeaveId: leaveId },
      order: { createdAt: "DESC" },
    });
    return new HttpSuccess(record);
  }

  @Post()
  public async newLeave(@Request() req: RequestWithUser, @Body() body: CreateProfileEmployeeLeave) {
    await new permission().PermissionUpdate(req, "SYS_REGISTRY_TEMP");
    if (!body.profileEmployeeId) {
      throw new HttpError(HttpStatus.BAD_REQUEST, "กรุณากรอก profileId");
    }

    const profile = await this.profileRepo.findOneBy({ id: body.profileEmployeeId });

    if (!profile) {
      throw new HttpError(HttpStatus.BAD_REQUEST, "ไม่พบ profile ดังกล่าว");
    }
    const leaveType = await this.leaveTypeRepository.findOne({
      where: { id: body.leaveTypeId },
    });
    if (!leaveType) {
      throw new HttpError(HttpStatus.NOT_FOUND, "ไม่พบข้อมูลประเภทลานี้");
    }
    const before = null;
    const data = new ProfileLeave();

    const meta = {
      createdUserId: req.user.sub,
      createdFullName: req.user.name,
      lastUpdateUserId: req.user.sub,
      lastUpdateFullName: req.user.name,
      createdAt: new Date(),
      lastUpdatedAt: new Date(),
    };

    Object.assign(data, { ...body, ...meta });
    const history = new ProfileLeaveHistory();
    Object.assign(history, { ...data, id: undefined });

    await this.leaveRepo.save(data);
    history.profileLeaveId = data.id;
    await this.leaveHistoryRepo.save(history);

    return new HttpSuccess(data.id);
  }

  @Patch("{leaveId}")
  public async editLeave(
    @Request() req: RequestWithUser,
    @Body() body: UpdateProfileLeave,
    @Path() leaveId: string,
  ) {
    await new permission().PermissionUpdate(req, "SYS_REGISTRY_TEMP");
    const record = await this.leaveRepo.findOneBy({ id: leaveId });

    if (!record) throw new HttpError(HttpStatus.NOT_FOUND, "ไม่พบข้อมูล");

    const leaveType = await this.leaveTypeRepository.findOne({
      where: { id: body.leaveTypeId },
    });
    if (!leaveType) {
      throw new HttpError(HttpStatus.NOT_FOUND, "ไม่พบข้อมูลประเภทลานี้");
    }
    const before = structuredClone(record);
    const history = new ProfileLeaveHistory();

    Object.assign(record, body);
    Object.assign(history, { ...record, id: undefined });

    history.profileLeaveId = leaveId;
    record.lastUpdateUserId = req.user.sub;
    record.lastUpdateFullName = req.user.name;
    record.lastUpdatedAt = new Date();
    history.lastUpdateUserId = req.user.sub;
    history.lastUpdateFullName = req.user.name;
    history.createdUserId = req.user.sub;
    history.createdFullName = req.user.name;
    history.createdAt = new Date();
    history.lastUpdatedAt = new Date();

    await Promise.all([
      this.leaveRepo.save(record),
      setLogDataDiff(req, { before, after: record }),
      this.leaveHistoryRepo.save(history),
    ]);

    return new HttpSuccess();
  }

  /**
   * API ลบข้อมูลการลา
   * @summary API ลบข้อมูลการลา
   * @param leaveId คีย์การลา
   */
  @Patch("update-delete/{leaveId}")
  public async updateIsDeleted(
    @Request() req: RequestWithUser,
    @Path() leaveId: string,
  ) {
    const record = await this.leaveRepo.findOneBy({ id: leaveId });
    if (!record) throw new HttpError(HttpStatus.NOT_FOUND, "ไม่พบข้อมูล");
    if (record.isDeleted === true) {
      return new HttpSuccess();
    }
    await new permission().PermissionDelete(req, "SYS_REGISTRY_TEMP");
    const before = structuredClone(record);
    const history = new ProfileLeaveHistory();
    const now = new Date();
    record.isDeleted = true;
    record.lastUpdateUserId = req.user.sub;
    record.lastUpdateFullName = req.user.name;
    record.lastUpdatedAt = now;

    Object.assign(history, { ...record, id: undefined });
    history.profileLeaveId = leaveId;
    history.createdUserId = req.user.sub;
    history.createdFullName = req.user.name;
    history.createdAt = now;

    await Promise.all([
      this.leaveRepo.save(record, { data: req }),
      setLogDataDiff(req, { before, after: record }),
      this.leaveHistoryRepo.save(history, { data: req }),
    ]);

    return new HttpSuccess();
  }

  @Delete("{leaveId}")
  public async deleteLeave(@Path() leaveId: string, @Request() req: RequestWithUser) {
    await new permission().PermissionDelete(req, "SYS_REGISTRY_TEMP");
    await this.leaveHistoryRepo.delete({
      profileLeaveId: leaveId,
    });

    const result = await this.leaveRepo.delete({ id: leaveId });

    if (result.affected == undefined || result.affected <= 0) {
      throw new HttpError(HttpStatus.NOT_FOUND, "ไม่พบข้อมูล");
    }

    return new HttpSuccess();
  }
}
